package com.loit.common.langfang;

import com.loit.common.langfang.dto.CommandManualDTO;
import com.loit.common.langfang.dto.LimitUnitType;
import com.loit.common.utils.DateUtils;
import com.loit.common.utils.ListUtil;
import com.loit.common.utils.StringUtils;
import com.loit.common.utils.excel.ImportExcel;
import com.loit.common.utils.file.FileUtils;
import com.loit.common.utils.freemarker.FreeMarkerUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.entity.ContentType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
public class GeneratorCommandManualScript {


    private static Logger logger = LoggerFactory.getLogger(GeneratorCommandManualScript.class);

    //TODO  1：事件，2：部件
    public static final String CASE_TYPE = "2";
    //TODO 大类id 需要手动设置下,后面可以改成查询数据库
    public static final String BIG_TYPE_ID = "344";


    protected static String root_path = "F:\\9Git140\\loit-build-common\\loit-build-component\\loit-build-deploy-env\\src\\main\\resources\\langfang";


    public static void main(String[] args) {

        try {

            String filePathStr = "F:\\9Git140\\loit-build-common\\loit-build-component\\loit-build-deploy-env\\src\\main\\resources\\井盖大小类扩展20230516.xlsx";

            File pdfFile = new File(filePathStr);
            FileInputStream fileInputStream = new FileInputStream(pdfFile);
            MultipartFile multipartFile = new MockMultipartFile(pdfFile.getName(), pdfFile.getName(),

                    ContentType.APPLICATION_OCTET_STREAM.toString(), fileInputStream);
            ImportExcel ei = new ImportExcel(multipartFile, 2, 0);
            List<CommandManualDTO> commandManualDTOList = ei.getDataList(CommandManualDTO.class);

            if (ListUtil.isEmpty(commandManualDTOList)) {
                return;
            }

            List<CommandManualDTO> transformList = transform(commandManualDTOList);

            createBigTypeScript(transformList);

            createSmallTypeScript(transformList);

            createManageProblemScript(transformList);

            createManageProblemSmallTypeScript(transformList);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }


    /**
     * 生成大类脚本
     *
     * @throws IOException
     */
    private static void createBigTypeScript(List<CommandManualDTO> commandManualList) {


        List<CommandManualDTO> bitTypeList = commandManualList.stream().collect(Collectors.collectingAndThen(Collectors.toCollection(() ->
                new TreeSet<>(Comparator.comparing(CommandManualDTO::getBigTypeCode))), ArrayList::new));

        if (bitTypeList == null || bitTypeList.isEmpty()) {
            return;
        }

        bitTypeList.sort(Comparator.comparing(CommandManualDTO::getBigTypeCodeInt));


        for (CommandManualDTO commandManualDTO : bitTypeList) {


            String sql1 = "select * from base_base_casetype order by parent_id asc;\n";
            String sql2 = "select * from base_base_casetype where obj_name = '" + commandManualDTO.getBigTypeName() + "';\n";

            Map model = new HashMap();
            model.put("bigTypeCode", commandManualDTO.getBigTypeCode());
            model.put("bigTypeName", commandManualDTO.getBigTypeName());
            model.put("caseType", CASE_TYPE);


            String configFilePath = root_path + "\\1、bigTypeScript.sql";
            String jobConfig = FreeMarkerUtils.process("langfang" + "\\bigTypeScript.ftl", model);
            FileUtils.append(configFilePath, sql1);
            FileUtils.append(configFilePath, sql2);
            FileUtils.append(configFilePath, jobConfig);
        }
    }


    /**
     * 生成小类脚本
     *
     * @throws IOException
     */
    private static void createSmallTypeScript(List<CommandManualDTO> commandManualList) {


        List<CommandManualDTO> smallTypeList = commandManualList.stream().collect(Collectors.collectingAndThen(Collectors.toCollection(() ->
                new TreeSet<>(Comparator.comparing(CommandManualDTO::getBigAndSmallCode))), ArrayList::new));

        if (smallTypeList == null || smallTypeList.isEmpty()) {
            return;
        }

        smallTypeList.sort(Comparator.comparing(CommandManualDTO::getBigTypeCodeInt)
                .thenComparing(CommandManualDTO::getSmallTypeCodeInt));

        String configFilePath = root_path + "\\2、smallTypeScript.sql";

        CommandManualDTO commandManual = smallTypeList.get(0);
        String sql = "select * from base_base_casetype where parent_id = '" + commandManual.getBigTypeCode() + "';\n";
        FileUtils.append(configFilePath, sql);

        for (CommandManualDTO commandManualDTO : smallTypeList) {
            Map model = new HashMap();
            model.put("bigTypeId", commandManualDTO.getBigTypeId());
            model.put("bigTypeCode", commandManualDTO.getBigTypeCode());
            model.put("bigTypeName", commandManualDTO.getBigTypeName());
            model.put("smallTypeCode", commandManualDTO.getSmallTypeCode());
            model.put("smallTypeName", commandManualDTO.getSmallTypeName());

            model.put("caseType", CASE_TYPE);


            String jobConfig = FreeMarkerUtils.process("langfang" + "\\smallTypeScript.ftl", model);
            FileUtils.append(configFilePath, jobConfig);
        }
    }


    /**
     * 生成管理问题脚本
     *
     * @throws IOException
     */
    private static void createManageProblemScript(List<CommandManualDTO> commandManualList) {


        if (commandManualList == null || commandManualList.isEmpty()) {
            return;
        }


        for (CommandManualDTO commandManualDTO : commandManualList) {

            Map model = new HashMap();
            model.put("bigTypeId", commandManualDTO.getBigTypeId());
            model.put("bigTypeCode", commandManualDTO.getBigTypeCode());
            model.put("bigTypeName", commandManualDTO.getBigTypeName());
            model.put("smallTypeCode", commandManualDTO.getSmallTypeCode());
            model.put("smallTypeName", commandManualDTO.getSmallTypeName());
            model.put("manageProblem", commandManualDTO.getManageProblem());
            model.put("datetime", DateUtils.getDateTime());
            model.put("caseType", CASE_TYPE);

            LimitUnitType alimitUnit = getLimitUnit(commandManualDTO.getAtype());
            LimitUnitType blimitUnit = getLimitUnit(commandManualDTO.getBtype());
            LimitUnitType climitUnit = getLimitUnit(commandManualDTO.getCtype());

            model.put("alimit", alimitUnit.getLimit());
            model.put("aunit", alimitUnit.getUnit());

            model.put("blimit", blimitUnit.getLimit());
            model.put("bunit", blimitUnit.getUnit());

            model.put("climit", climitUnit.getLimit());
            model.put("cunit", climitUnit.getUnit());

            String configFilePath = root_path + "\\3、manageProblemScript.sql";
            String jobConfig = FreeMarkerUtils.process("langfang" + "\\manageProblemScript.ftl", model);
            FileUtils.append(configFilePath, jobConfig);
        }
    }


    /**
     * 生成管理问题脚本-清洗小类id
     *
     * @throws IOException
     */
    private static void createManageProblemSmallTypeScript(List<CommandManualDTO> commandManualList) {


        if (commandManualList == null || commandManualList.isEmpty()) {
            return;
        }


        for (CommandManualDTO commandManualDTO : commandManualList) {

            Map model = new HashMap();
            model.put("bigTypeId", commandManualDTO.getBigTypeId());
            model.put("bigTypeCode", commandManualDTO.getBigTypeCode());
            model.put("bigTypeName", commandManualDTO.getBigTypeName());
            model.put("smallTypeCode", commandManualDTO.getSmallTypeCode());
            model.put("smallTypeName", commandManualDTO.getSmallTypeName());
            model.put("manageProblem", commandManualDTO.getManageProblem());


            String configFilePath = root_path + "\\4、manageProblem_smallTypeId_Script.sql";
            String jobConfig = FreeMarkerUtils.process("langfang" + "\\manageProblem_smallTypeId_Script.ftl", model);
            FileUtils.append(configFilePath, jobConfig);
        }
    }


    private static LimitUnitType getLimitUnit(String limitStr) {

        LimitUnitType limitUnitType = new LimitUnitType();
        if (StringUtils.isEmpty(limitStr)) {
            logger.error("limitStr is null");
            return limitUnitType;
        }
        limitStr = limitStr.trim();


        //0：工作日,1：工作时, 2：紧急工作时
        if (limitStr.contains("工作日")) {
            limitUnitType.setUnit("0");
            limitUnitType.setLimit(limitStr.replaceAll("工作日", "").trim());
        } else if (limitStr.contains("紧急工作时")) {
            limitUnitType.setUnit("2");
            limitUnitType.setLimit(limitStr.replaceAll("紧急工作时", "").trim());
        } else if (limitStr.contains("工作时")) {
            limitUnitType.setUnit("1");
            limitUnitType.setLimit(limitStr.replaceAll("工作时", "").trim());
        }

        if (limitUnitType.getLimit().length() > 1) {
            logger.error("limit length > 1" + limitUnitType.getLimit());
        }

        return limitUnitType;
    }



    /**
     * 对excel中的数据做转换:
     * 1、去除空数据
     * 2、填充大类小类
     * 3、填充A类B类C类处置时限一样的
     * 4、
     */
    private static List<CommandManualDTO> transform(List<CommandManualDTO> commandManualDTOList) {

        String bigTypeCode = null;
        String bigTypeName = null;
        String smallTypeCode = null;
        String smallTypeName = null;

        List<CommandManualDTO> resultList = new ArrayList();

        for (CommandManualDTO commandManual : commandManualDTOList) {
            if (StringUtils.isEmpty(commandManual.getManageProblem())) {
                continue;
            }


            if (StringUtils.isNotEmpty(commandManual.getBigTypeCode())) {
                bigTypeCode = commandManual.getBigTypeCode();
            } else {
                commandManual.setBigTypeCode(bigTypeCode);
            }


            if (StringUtils.isNotEmpty(commandManual.getBigTypeName())) {
                bigTypeName = commandManual.getBigTypeName();
            } else {
                commandManual.setBigTypeName(bigTypeName);
            }


            if (StringUtils.isNotEmpty(commandManual.getSmallTypeCode())) {
                smallTypeCode = commandManual.getSmallTypeCode();
            } else {
                commandManual.setSmallTypeCode(smallTypeCode);
            }

            if (StringUtils.isNotEmpty(commandManual.getSmallTypeName())) {
                smallTypeName = commandManual.getSmallTypeName();
            } else {
                commandManual.setSmallTypeName(smallTypeName);
            }


            String atype = commandManual.getAtype();
            if (StringUtils.isEmpty(commandManual.getBtype())) {
                commandManual.setBtype(atype);
            }

            if (StringUtils.isEmpty(commandManual.getCtype())) {
                commandManual.setCtype(atype);
            }

            commandManual.setBigAndSmallCode(commandManual.getBigTypeCode() + "|" + commandManual.getSmallTypeCode());


            commandManual.setBigTypeCodeInt(Integer.valueOf(commandManual.getBigTypeCode()));
            commandManual.setSmallTypeCodeInt(Integer.valueOf(commandManual.getSmallTypeCode()));


            //TODO　查询从数据库查询
            commandManual.setBigTypeId(BIG_TYPE_ID);
            resultList.add(commandManual);

        }

        return resultList;
    }





}
