package com.loit.common.script;

import com.loit.common.script.dto.BackendFrontEnum;
import com.loit.common.script.dto.BuildEnvEnum;
import com.loit.common.script.dto.DeployInfoDataDTO;
import com.loit.common.script.dto.FrontInstanceDto;
import com.loit.common.utils.ListUtil;
import com.loit.common.utils.StringUtils;
import com.loit.common.utils.excel.ImportExcel;
import com.loit.common.utils.file.FileUtils;
import com.loit.common.utils.freemarker.FreeMarkerUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.entity.ContentType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

@Slf4j
public class GeneratorDeployJobScript {

    protected static String root_path = "F:\\9Git140\\loit-build-common\\loit-build-component\\loit-build-deploy-env\\src\\main\\resources\\bin";

    /**
     * 构建环境 生产或者 准生产
     * TODO 构建前需修改环境
     */
    private static BuildEnvEnum buildEnvEnum = BuildEnvEnum.PRE_PRODUCT;

    protected static String root_path_jenkins_full = root_path + "\\jenkinsDeploy";


    public static void main(String[] args) {

        try {
            //String filePathStr = Thread.currentThread().getContextClassLoader().getResource("deployInfo.xlsx").getPath();
            String filePathStr = "F:\\9Git140\\loit-build-common\\loit-build-component\\loit-build-deploy-env\\src\\main\\resources\\deployInfo.xlsx";

            File pdfFile = new File(filePathStr);
            FileInputStream fileInputStream = new FileInputStream(pdfFile);
            MultipartFile multipartFile = new MockMultipartFile(pdfFile.getName(), pdfFile.getName(),

                    ContentType.APPLICATION_OCTET_STREAM.toString(), fileInputStream);
            ImportExcel ei = new ImportExcel(multipartFile, 1, 0);
            List<DeployInfoDataDTO> grayVersionDataDTOList = ei.getDataList(DeployInfoDataDTO.class);

            if (ListUtil.isEmpty(grayVersionDataDTOList)) {
                return;
            }

            for (DeployInfoDataDTO grayVersionDataDTO : grayVersionDataDTOList) {

                String needCreateInstanceSql = grayVersionDataDTO.getNeedCreateInstanceSql();
                if ("no".equals(needCreateInstanceSql)) {
                    continue;
                }

                buildInitEnv(grayVersionDataDTO);
            }

        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    /**
     * 生成InitEnv
     *
     * @throws IOException
     */
    private static void buildInitEnv(DeployInfoDataDTO grayVersionDataDTO) {

        String fileDir = root_path_jenkins_full + "\\" + buildEnvEnum.getCode();

        File folder = new File(fileDir);
        if (!folder.exists()) {
            folder.mkdirs();
        }

        String serviceName = grayVersionDataDTO.getServiceName();
        String fixedIp = grayVersionDataDTO.getFixedIp();
        String floatingIp = grayVersionDataDTO.getFloatingIp();
        String deployJar = grayVersionDataDTO.getDeployJar();
        String port = grayVersionDataDTO.getPort();
        String deployPath = grayVersionDataDTO.getDeployPath();
        if (StringUtils.isBlank(serviceName)) {
            return;
        }

        String createJobFile = root_path_jenkins_full + "\\" + buildEnvEnum.getCode() + "\\createJob.txt";

        String jenkinsJobName = createJenkinsJobName(grayVersionDataDTO);
        String deployShFileName = "deploy-" + serviceName + "-" + port + ".sh";

        Map model = new HashMap();
        model.put("deployJobName", jenkinsJobName);
        model.put("serviceName", serviceName);
        model.put("deployPath", deployPath);
        model.put("deployShFileName", deployShFileName);
        model.put("fixedIp", fixedIp);
        model.put("floatingIp", floatingIp);
        model.put("deployJar", deployJar);

        String result = FreeMarkerUtils.process("deploy" + "\\" + buildEnvEnum.getCode() + "\\jenkinsJobCreate.ftl", model);
        FileUtils.appendNewLine(createJobFile, result);

        String jobsDir = root_path_jenkins_full + "\\" + buildEnvEnum.getCode() + "\\" + jenkinsJobName;
        File jobsFolder = new File(jobsDir);
        if (!jobsFolder.exists()) {
            jobsFolder.mkdirs();
        }
        BackendFrontEnum backendFrontType = BackendFrontEnum.getEnumByCode(grayVersionDataDTO.getBackendFrontType());
        if (BackendFrontEnum.BACKEND.equals(backendFrontType)) {
            String configFilePath = jobsDir + "\\config.xml";
            String jobConfig = FreeMarkerUtils.process("deploy" + "\\" + buildEnvEnum.getCode() + "\\jobConfigBackend.ftl", model);
            FileUtils.appendNewLine(configFilePath, jobConfig);
        } else {

            String configFilePath = jobsDir + "\\config.xml";
            String jobConfig = FreeMarkerUtils.process("deploy" + "\\" + buildEnvEnum.getCode() + "\\jobConfigFront.ftl", model);
            FileUtils.appendNewLine(configFilePath, jobConfig);
        }

    }

    private static String createJenkinsJobName(DeployInfoDataDTO grayVersionDataDTO) {

        String serviceName = grayVersionDataDTO.getServiceName();
        String floatingIp = grayVersionDataDTO.getFloatingIp();
        String fixedIp = grayVersionDataDTO.getFixedIp();
        String port = grayVersionDataDTO.getPort();

        if ("getway".equals(serviceName)) {
            serviceName = "gateway";
        }


        BackendFrontEnum backendFrontType = BackendFrontEnum.getEnumByCode(grayVersionDataDTO.getBackendFrontType());

        String[] fIps = floatingIp.split("\\.");
        String fIp = fIps[3];

        String[] fixIps = fixedIp.split("\\.");
        String fixIp = fixIps[3];

        String jenkinsJobName = "";
        if (BackendFrontEnum.BACKEND.equals(backendFrontType)) {
            jenkinsJobName = "loit-deploy-" + serviceName + "-" + port + "-" + fIp + "-" + fixIp;
        }

        if (!BackendFrontEnum.BACKEND.equals(backendFrontType)) {
            jenkinsJobName = "loit-front-deploy-" + serviceName + "-" + port + "-" + fIp + "-" + fixIp;
        }
        return jenkinsJobName;
    }


}
