package com.loit.component.keygen.leaf.core.reg;

import com.loit.component.keygen.leaf.core.listener.DataChangedEventListener;
import com.loit.component.keygen.leaf.core.spi.TypeBasedSPI;

import java.util.List;

/**
 * Registry center.
 */
public interface RegistryCenter extends TypeBasedSPI {
    
    /**
     * Initialize registry center.
     * 
     * @param config registry center configuration
     */
    void init(RegistryCenterConfiguration config);
    
    /**
     * Get data from registry center.
     * 
     * <p>Maybe use cache if existed.</p>
     * 
     * @param key key of data
     * @return value of data
     */
    String get(String key);
    
    /**
     * Get data from registry center directly.
     * 
     * <p>Cannot use cache.</p>
     *
     * @param key key of data
     * @return value of data
     */
    String getDirectly(String key);
    
    /**
     * Judge data is existed or not.
     * 
     * @param key key of data
     * @return data is existed or not
     */
    boolean isExisted(String key);
    
    /**
     * Get node's sub-nodes list.
     *
     * @param key key of data
     * @return sub-nodes name list
     */
    List<String> getChildrenKeys(String key);
    
    /**
     * Persist data.
     * 
     * @param key key of data
     * @param value value of data
     */
    void persist(String key, String value);
    
    /**
     * Update data.
     *
     * @param key key of data
     * @param value value of data
     */
    void update(String key, String value);
    
    /**
     * Persist ephemeral data.
     *
     * @param key key of data
     * @param value value of data
     */
    void persistEphemeral(String key, String value);
    
    /**
     * Watch key or path of the registry.
     *
     * @param key key of data
     * @param dataChangedEventListener data changed event listener
     */
    void watch(String key, DataChangedEventListener dataChangedEventListener);
    
    /**
     * Close.
      */
    void close();

    /**
     * Initialize the lock of the key.
     *
     * @param key key of data
     */
    void initLock(String key);

    /**
     * Try to get the lock of the key.
     *
     * @return get the lock or not
     */
    boolean tryLock();

    /**
     * Try to release the lock of the key.
     *
     */
    void tryRelease();
}
