package com.loit.component.keygen.leaf.zk;

import com.loit.component.keygen.leaf.core.reg.RegistryCenter;
import com.loit.component.keygen.leaf.core.reg.RegistryCenterConfiguration;
import com.loit.component.keygen.leaf.zk.util.EmbedTestingServer;
import org.junit.BeforeClass;
import org.junit.Test;

import java.util.List;
import java.util.Properties;

import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.assertThat;

public class CuratorZookeeperRegistryCenterTest {
    
    private static RegistryCenter curatorZookeeperRegistryCenter = new CuratorZookeeperRegistryCenter();
    
    @BeforeClass
    public static void init() {
        EmbedTestingServer.start();
        RegistryCenterConfiguration configuration = new RegistryCenterConfiguration(curatorZookeeperRegistryCenter.getType(), new Properties());
        configuration.setServerLists("127.0.0.1:2181");
        curatorZookeeperRegistryCenter.init(configuration);
    }
    
    @Test
    public void assertPersist() {
        curatorZookeeperRegistryCenter.persist("/test", "value1");
        assertThat(curatorZookeeperRegistryCenter.get("/test"), is("value1"));
    }
    
    @Test
    public void assertUpdate() {
        curatorZookeeperRegistryCenter.persist("/test", "value2");
        assertThat(curatorZookeeperRegistryCenter.get("/test"), is("value2"));
    }
    
    @Test
    public void assertPersistEphemeral() {
        curatorZookeeperRegistryCenter.persist("/test/ephemeral", "value3");
        assertThat(curatorZookeeperRegistryCenter.get("/test/ephemeral"), is("value3"));
    }
    
    @Test
    public void assertGetDirectly() {
        curatorZookeeperRegistryCenter.persist("/test", "value4");
        assertThat(curatorZookeeperRegistryCenter.getDirectly("/test"), is("value4"));
    }
    
    @Test
    public void assertIsExisted() {
        curatorZookeeperRegistryCenter.persist("/test/existed", "value5");
        assertThat(curatorZookeeperRegistryCenter.isExisted("/test/existed"), is(true));
    }
    
    @Test
    public void assertGetChildrenKeys() {
        curatorZookeeperRegistryCenter.persist("/test/children/1", "value11");
        curatorZookeeperRegistryCenter.persist("/test/children/2", "value12");
        curatorZookeeperRegistryCenter.persist("/test/children/3", "value13");
        List<String> childrenKeys = curatorZookeeperRegistryCenter.getChildrenKeys("/test/children");
        assertThat(childrenKeys.size(), is(3));
    }
    
    @Test
    public void assertLock() {
        curatorZookeeperRegistryCenter.initLock("/test/lock1");
        assertThat(curatorZookeeperRegistryCenter.tryLock(), is(true));
    }
    
    @Test
    public void assertRelease() {
        curatorZookeeperRegistryCenter.initLock("/test/lock2");
        curatorZookeeperRegistryCenter.tryLock();
        curatorZookeeperRegistryCenter.tryRelease();
    }
    
    @Test(expected = IllegalMonitorStateException.class)
    public void assertReleaseWithoutLock() {
        curatorZookeeperRegistryCenter.initLock("/test/lock3");
        curatorZookeeperRegistryCenter.tryRelease();
    }
}
